@@ -1,16 +1,14 @@
 from dropbox.client import DropboxClient
 from dropbox.session import DropboxSession
-from flask import request, session as flask_session, url_for
-from werkzeug.utils import cached_property
-
-try:
-    # For Dropbox SDK 1.4.1+
-    from dropbox.session import OAuthToken
-except ImportError:
-    from oauth.oauth import OAuthToken
+from flask import g, request, session as flask_session, url_for
+from werkzeug.utils import cached_property, import_string
 
 from .blueprint import DropboxBlueprint
-from .settings import DROPBOX_ACCESS_TOKEN_KEY, DROPBOX_REQUEST_TOKEN_KEY
+from .compat import OAuthToken
+from .settings import (
+    ACCOUNT_INFO_CACHE_KEY, CLIENT_CACHE_KEY, DROPBOX_ACCESS_TOKEN_KEY,
+    DROPBOX_REQUEST_TOKEN_KEY, SESSION_CACHE_KEY
+)
 
 
 __all__ = ('Dropbox', )
@@ -18,7 +16,8 @@ __all__ = ('Dropbox', )
 
 DROPBOX_CONFIGS = ('*DROPBOX_KEY', '*DROPBOX_SECRET', '*DROPBOX_ACCESS_TYPE',
                    'DROPBOX_CALLBACK_TEMPLATE', 'DROPBOX_CALLBACK_URL',
-                   'DROPBOX_LOGIN_REDIRECT', 'DROPBOX_LOGOUT_REDIRECT')
+                   'DROPBOX_LOGIN_REDIRECT', 'DROPBOX_LOGOUT_REDIRECT',
+                   'DROPBOX_CACHE_STORAGE')
 
 
 class Dropbox(object):
@@ -40,25 +39,39 @@ class Dropbox(object):
 
         Also stores result in instance cache to reduce network connections.
         """
-        if not hasattr(self, '_account_info_cache'):
+        if not hasattr(self.cache_storage, ACCOUNT_INFO_CACHE_KEY):
             account_info = self.client.account_info()
-            setattr(self, '_account_info_cache', account_info)
-        return getattr(self, '_account_info_cache')
+            setattr(self.cache_storage, ACCOUNT_INFO_CACHE_KEY, account_info)
+        return getattr(self.cache_storage, ACCOUNT_INFO_CACHE_KEY)
+
+    @cached_property
+    def cache_storage(self):
+        """
+        Cache storage. For versions 0.2 this storage was self, for later
+        versions it should be configurable, but ``flask.g`` by default.
+        """
+        if self.DROPBOX_CACHE_STORAGE:
+            if self.DROPBOX_CACHE_STORAGE == 'self':
+                return self
+            return (import_string(self.DROPBOX_CACHE_STORAGE)
+                    if isinstance(self.DROPBOX_CACHE_STORAGE, basestring)
+                    else self.DROPBOX_CACHE_STORAGE)
+        return g
 
     @property
     def client(self):
         """
         Initialize Dropbox client instance or return it from instance cache.
         """
-        if not hasattr(self, '_client_cache'):
+        if not hasattr(self.cache_storage, CLIENT_CACHE_KEY):
             assert self.is_authenticated, 'Please, login with Dropbox first.'
-            token = flask_session[DROPBOX_ACCESS_TOKEN_KEY]
 
-            self.session.set_token(token.key, token.secret)
+            key, secret = flask_session[DROPBOX_ACCESS_TOKEN_KEY]
+            self.session.set_token(key, secret)
             client = DropboxClient(self.session)
 
-            setattr(self, '_client_cache', client)
-        return getattr(self, '_client_cache')
+            setattr(self.cache_storage, CLIENT_CACHE_KEY, client)
+        return getattr(self.cache_storage, CLIENT_CACHE_KEY)
 
     def init_app(self, app):
         """
@@ -90,8 +103,7 @@ class Dropbox(object):
         """
         Check if current user logged in with Dropbox or not.
         """
-        db_session_key = flask_session.get(DROPBOX_ACCESS_TOKEN_KEY)
-        return db_session_key and isinstance(db_session_key, OAuthToken)
+        return DROPBOX_ACCESS_TOKEN_KEY in flask_session
 
     def login(self, request_token):
         """
@@ -99,7 +111,8 @@ class Dropbox(object):
         """
         # Generate access token for current user
         access_token = self.session.obtain_access_token(request_token)
-        flask_session[DROPBOX_ACCESS_TOKEN_KEY] = access_token
+        flask_session[DROPBOX_ACCESS_TOKEN_KEY] = [access_token.key,
+                                                   access_token.secret]
 
         # Remove available request token
         del flask_session[DROPBOX_REQUEST_TOKEN_KEY]
@@ -122,13 +135,12 @@ class Dropbox(object):
         """
         Revoke access for Dropbox user to the site.
         """
-        # Do not forget to remove account info and client cache as well
-        if hasattr(self, '_account_info_cache'):
-            delattr(self, '_account_info_cache')
-
-        if hasattr(self, '_client_cache'):
-            delattr(self, '_client_cache')
-
+        if hasattr(self.cache_storage, ACCOUNT_INFO_CACHE_KEY):
+            delattr(self.cache_storage, ACCOUNT_INFO_CACHE_KEY)
+        if hasattr(self.cache_storage, CLIENT_CACHE_KEY):
+            delattr(self.cache_storage, CLIENT_CACHE_KEY)
+        if hasattr(self.cache_storage, SESSION_CACHE_KEY):
+            delattr(self.cache_storage, SESSION_CACHE_KEY)
         if DROPBOX_ACCESS_TOKEN_KEY in flask_session:
             del flask_session[DROPBOX_ACCESS_TOKEN_KEY]
 
@@ -151,18 +163,22 @@ class Dropbox(object):
         """
         Generate Dropbox session request token and place it to Flask session.
         """
-        token = self.session.obtain_request_token()
-        flask_session[DROPBOX_REQUEST_TOKEN_KEY] = token
-        return token
+        request_token = self.session.obtain_request_token()
+        flask_session[DROPBOX_REQUEST_TOKEN_KEY] = [request_token.key,
+                                                    request_token.secret]
+        return request_token
 
-    @cached_property
+    @property
     def session(self):
         """
         Initialize or return already initialized ``DropboxSession`` instance.
         """
-        return DropboxSession(self.DROPBOX_KEY,
-                              self.DROPBOX_SECRET,
-                              self.DROPBOX_ACCESS_TYPE)
+        if not hasattr(self.cache_storage, SESSION_CACHE_KEY):
+            session = DropboxSession(self.DROPBOX_KEY,
+                                     self.DROPBOX_SECRET,
+                                     self.DROPBOX_ACCESS_TYPE)
+            setattr(self.cache_storage, SESSION_CACHE_KEY, session)
+        return getattr(self.cache_storage, SESSION_CACHE_KEY)
 
 
 # Monkey-patch Dropbox SDK ``OAuthToken`` class to add support of pickling
