@@ -1,7 +1,7 @@
 from dropbox.rest import ErrorResponse
-from flask import current_app, redirect, render_template, request, session, \
-    url_for
+from flask import current_app, redirect, render_template, request, session
 
+from .compat import OAuthToken
 from .settings import DROPBOX_REQUEST_TOKEN_KEY
 from .utils import safe_url_for
 
@@ -32,15 +32,18 @@ def callback():
         return render_template(template, error_oauth_token=True)
 
     # oAuth token **should** be equal to stored request token
-    request_token = session.get(DROPBOX_REQUEST_TOKEN_KEY)
+    try:
+        key, secret = session.get(DROPBOX_REQUEST_TOKEN_KEY) or (None, None)
+    except ValueError:
+        return render_template(template, error_request_token=True)
 
-    if not request_token or oauth_token != request_token.key:
+    if oauth_token != key:
         return render_template(template, error_not_equal_tokens=True)
 
     # Do login with current request token
     try:
-        dropbox.login(request_token)
-    except ErrorResponse, e:
+        dropbox.login(OAuthToken(key, secret))
+    except ErrorResponse as e:
         return render_template(template, error_response=True, error=e)
 
     # Redirect to resulted page
